# -*- coding: utf-8 -*-
#
# Python demo program for executing an update of the
# RemoSens firmware
# 
# 'RemoSens' is a LAN/WiFi interface-box for legacy
# R&S USB power sensors
# 
# This programming example asks for a filename containing a
# new firmware for the RemoSens interface box
# 
# Communication is preferably done via TCP/IP VISA HiSLIP connection
# (although also SCPI-RAW communication is also supported by 'RemoSens')
# ___________________________________________________________________________
#
# To use this demo program, you need to have install the following:
#
#  Python 3.x        https://python.org/download/
#
#  PyVISA >= 1.8     https://github.com/pyvisa/pyvisa
#
#
#  +-- MEMORY AID -------------------------------------------------------------------------------------------+
#  |                                                                                                         |
#  |  How to install or upgrade a Python module XXXXX                                                        |
#  |                                                                                                         |
#  |  --> In case of a direct Internet connection:                                                           |
#  |                                                                                                         |
#  |         python -m pip install --upgrade XXXXX                                                           |
#  |                                                                                                         |
#  |      Example:                                                                                           |
#  |         python -m pip install --upgrade pyvisa                                                          |
#  |                                                                                                         |
#  |                                                                                                         |
#  |  --> In case you are behind a PROXY                                                                     |
#  |         (replace 'user', 'password', 'proxy-host.example.com' and 'port' with your locally valid data): |
#  |                                                                                                         |
#  |         set http_proxy=http://user:password@proxy-host.example.com:port                                 |
#  |         set https_proxy=http://user:password@proxy-host.example.com:port                                |
#  |         python -m pip install --upgrade XXXXX                                                           |
#  |                                                                                                         |
#  +---------------------------------------------------------------------------------------------------------+
#
# ____________________________________________________________________________
#
#  Author            Juergen D. Geltinger
#                    IFD - Ing.-Buero fuer Datentechnik
#                    Eugenbacher Str. 93a
#                    DE-84032 Altdorf
#                    Germany
#
#  Support           support@ifd.de
#
#  History           2022-02-18   JDG  Added optional backend-parameter;
#                    2022-02-25   JDG  Initial release
#
# ____________________________________________________________________________

#import os
import pyvisa as visa
from time import sleep
from configparser import ConfigParser
from os.path import exists


if __name__ == "__main__":
    # Get a possibly already defined host or IP from a local INI file
    # ---------------------------------------------------------------
    config = ConfigParser()
    config.read('FirmwareUpdate.ini')
    strCurrentHost = config.get('Main', 'HostOrIP',  fallback = '192.168.1.200')
    strNewFirmware = config.get('Main', 'NewFWFile', fallback = 'fw1.7.41.remo')
    # ---------------------------------------------------------------

    # Get HOST from User
    # ------------------
    if not strCurrentHost:
        strPrompt = "Enter Hostname or IP address [<none>]: "
    else:
        strPrompt = "Enter Hostname or IP address [" + strCurrentHost + "]: "
    devNameOrIP = input( strPrompt );
    if not devNameOrIP and strCurrentHost:
        devNameOrIP = strCurrentHost

    if not devNameOrIP and len(sys.argv) > 1:
      devNameOrIP = sys.argv[1]
    
    if not devNameOrIP:
        print( "Error: Device Name or IP address missing" );
        print( "  Please give the device's hostname or IP as a parameter" );
        print( "  Example:" );
        print( "    %s  192.168.1.200  ..." % sys.argv[0] );
        sys.exit(-1)
    # ------------------

    # Get FIRMWARE-FILENAME from User
    # -------------------------------
    if not strNewFirmware:
        strPrompt = "Enter Path/Filename of new FW [<none>]: "
    else:
        strPrompt = "Enter Path/Filename of new FW [" + strNewFirmware + "]: "
    pathNameFW = input( strPrompt );
    if not pathNameFW and strNewFirmware:
        pathNameFW = strNewFirmware

    if not pathNameFW and len(sys.argv) > 2:
      pathNameFW = sys.argv[2]
    
    if not pathNameFW:
        print( "Error: Path/Filename of new Firmware missing" );
        print( "  Please give the full path as a parameter" );
        print( "  Example:" );
        print( "    %s  192.168.1.200  fw1.7.41.remo" % sys.argv[0] );
        sys.exit(-1)


    # Save the user's parameters for a possibly later use
    # ----------------------------------------------------------------
    if not config.has_section('Main'):
        config.add_section('Main')
    config.set('Main', 'HostOrIP', devNameOrIP)
    config.set('Main', 'NewFWFile', pathNameFW)
    with open('FirmwareUpdate.ini', 'w') as f:
        config.write(f)
    # ----------------------------------------------------------------


    fwUpdateFile = pathNameFW

    if not exists( fwUpdateFile):
        print( "File %s is not accessible or does not exist." % fwUpdateFile)
        sys.exit(-1)


    # Defining a VISA resource, which is to be communicated with...
    # ----------------------------------------------------------------
    #
    # For Firmware-Update this can only be a VISA SCPI-RAW
    # communication (also known as SOCKET 5025)
    #
    # ----------------------------------------------------------------
    remosensResource = 'TCPIP::{}::5025::SOCKET'.format(devNameOrIP)

    # --------------------------------------------------------------------
    # Opening the VISA device connection to our RemoSens device (by the resource as defined above)
    #
    # Hint: optionally a certain 'backend' parameter can be specified.
    #       For example, if 'PyVISA-py' is to be used as a backend VISA
    #       implementation, call
    #
    #         box = visa.ResourceManager('@py')
    #
    # --------------------------------------------------------------------
    backend = ""    # "@py" for PyVISA-py

    rm = visa.ResourceManager(backend)
    remosens = rm.open_resource(remosensResource)

    # Very important for SCPI-RAW communication
    remosens.read_termination = '\n'
    remosens.write_termination = '\n'
    remosens.timeout = 20000


    # Information about current Interface-Box FW Version
    print("'RemoSens' Interface-Box Version: " + remosens.query("SYST:IFVersion?"))

    print("Updating 'RemoSens' firmware... ")

    # Assemble Firmware-Update command
    strCmd = "SYST:IFUP "
    fIn = open( fwUpdateFile, mode='rb')
    content = fIn.read()
    strCmd += content.decode('utf-8')

    # Here it is not necessary to add a LF, because we have already
    # defined the VISA.write_termination to be LF ('\n')
    #strCmd += "\n"    # Don't forget the LF delimiter

    # Sending the new firmware to the RemoSens Interface-Box;
    # this will start the firmware-update process...
    remosens.write( strCmd )
    sleep(1.0)

    # At the end of FW update, the RemoSens Interface-Box will
    # automatically reboot. Thus, the current communication session
    # will not be valid any longer
    remosens.close()

    nRemaining = 60
    while nRemaining > 0:
        print("Firmware Update in progress - {} s remaining...".format(nRemaining))
        sleep(1.0)
        nRemaining = nRemaining - 1

    # Re-open the communication channel...
    remosens = rm.open_resource(remosensResource)
    remosens.read_termination = '\n'
    remosens.write_termination = '\n'

    # Interface-Box FW Version after update
    print("'RemoSens' Interface-Box Version: " + remosens.query("SYST:IFVersion?"))

    # Close and finish program
    remosens.close()
    
